package java.security;

import java.util.concurrent.ConcurrentHashMap;
import java.util.Collections;
import java.util.Locale;
import java.util.HashSet;
import java.lang.reflect.Field;
import jdk.internal.event.EventHelper;
import sun.security.jca.GetInstance;
import java.util.Iterator;
import java.util.Set;
import java.util.LinkedHashSet;
import java.util.Hashtable;
import sun.security.jca.ProviderList;
import java.util.Enumeration;
import java.util.List;
import sun.security.jca.Providers;
import java.net.URL;
import java.io.File;
import sun.security.util.PropertyExpander;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.util.Map;
import java.util.Properties;
import sun.security.util.Debug;

public final class Security
{
    private static final Debug sdebug;
    private static Properties props;
    private static final Map<String, Class<?>> spiMap;
    
    private static void initialize() {
        Security.props = new Properties();
        boolean n = false;
        boolean n2 = false;
        File securityPropFile = securityPropFile("java.security");
        if (securityPropFile.exists()) {
            BufferedInputStream inStream = null;
            try {
                inStream = new BufferedInputStream(new FileInputStream(securityPropFile));
                Security.props.load(inStream);
                n = true;
                if (Security.sdebug != null) {
                    Security.sdebug.println("reading security properties file: " + securityPropFile);
                }
            }
            catch (IOException ex) {
                if (Security.sdebug != null) {
                    Security.sdebug.println("unable to load security properties from " + securityPropFile);
                    ex.printStackTrace();
                }
            }
            finally {
                if (inStream != null) {
                    try {
                        inStream.close();
                    }
                    catch (IOException ex2) {
                        if (Security.sdebug != null) {
                            Security.sdebug.println("unable to close input stream");
                        }
                    }
                }
            }
        }
        if ("true".equalsIgnoreCase(Security.props.getProperty("security.overridePropertiesFile"))) {
            String s = System.getProperty("java.security.properties");
            if (s != null && s.startsWith("=")) {
                n2 = true;
                s = s.substring(1);
            }
            if (!(n2 == 0)) {
                Security.props = new Properties();
                if (Security.sdebug != null) {
                    Security.sdebug.println("overriding other security properties files!");
                }
            }
            if (s != null) {
                Object inStream2 = null;
                try {
                    s = PropertyExpander.expand(s);
                    File o = new File(s);
                    Object obj;
                    if (o.exists()) {
                        obj = new URL("file:" + o.getCanonicalPath());
                    }
                    else {
                        obj = new URL(s);
                    }
                    inStream2 = new BufferedInputStream(obj.openStream());
                    Security.props.load(inStream2);
                    n = true;
                    if (Security.sdebug != null) {
                        Security.sdebug.println("reading security properties file: " + obj);
                        if (!(n2 == 0)) {
                            Security.sdebug.println("overriding other security properties files!");
                        }
                    }
                }
                catch (Exception ex3) {
                    if (Security.sdebug != null) {
                        Security.sdebug.println("unable to load security properties from " + s);
                        ex3.printStackTrace();
                    }
                }
                finally {
                    if (inStream2 != null) {
                        try {
                            inStream2.close();
                        }
                        catch (IOException ex4) {
                            if (Security.sdebug != null) {
                                Security.sdebug.println("unable to close input stream");
                            }
                        }
                    }
                }
            }
        }
        if (n == 0) {
            initializeStatic();
            if (Security.sdebug != null) {
                Security.sdebug.println("unable to load security properties -- using defaults");
            }
        }
    }
    
    private static void initializeStatic() {
        Security.props.put("security.provider.1", "sun.security.provider.Sun");
        Security.props.put("security.provider.2", "sun.security.rsa.SunRsaSign");
        Security.props.put("security.provider.3", "com.sun.net.ssl.internal.ssl.Provider");
        Security.props.put("security.provider.4", "com.sun.crypto.provider.SunJCE");
        Security.props.put("security.provider.5", "sun.security.jgss.SunProvider");
        Security.props.put("security.provider.6", "com.sun.security.sasl.Provider");
    }
    
    private Security() {
    }
    
    private static File securityPropFile(String str) {
        String separator = File.separator;
        return new File(System.getProperty("java.home") + separator + "lib" + separator + "security" + separator + str);
    }
    
    private static Security.ProviderProperty getProviderProperty(String s) {
        __Null _Null = null;
        List<Provider> providers = Providers.getProviderList().providers();
        int i = 0;
        while (i < providers.size()) {
            Provider provider = (Provider)providers.get(i);
            String className = provider.getProperty(s);
            if (className == null) {
                Enumeration<Object> keys = provider.keys();
                while (keys.hasMoreElements() && className == null) {
                    String anotherString = (String)keys.nextElement();
                    if (!s.equalsIgnoreCase(anotherString)) {
                        continue;
                    }
                    className = provider.getProperty(anotherString);
                    break;
                }
            }
            if (className != null) {
                Security.ProviderProperty o = new Security.ProviderProperty();
                o.className = className;
                o.provider = provider;
                return o;
            }
            ++i;
        }
        return _Null;
    }
    
    private static String getProviderProperty(String s, Provider provider) {
        String s2 = provider.getProperty(s);
        if (s2 == null) {
            Enumeration<Object> keys = provider.keys();
            while (keys.hasMoreElements() && s2 == null) {
                String anotherString = (String)keys.nextElement();
                if (s.equalsIgnoreCase(anotherString)) {
                    s2 = provider.getProperty(anotherString);
                    break;
                }
                continue;
            }
        }
        return s2;
    }
    
    @Deprecated
    public static String getAlgorithmProperty(String str, String str2) {
        Security.ProviderProperty providerProperty = getProviderProperty("Alg." + str2 + "." + str);
        if (providerProperty != null) {
            return providerProperty.className;
        }
        return null;
    }
    
    public static synchronized int insertProviderAt(Provider p2, int n) {
        String name = p2.getName();
        checkInsertProvider(name);
        ProviderList fullProviderList = Providers.getFullProviderList();
        ProviderList insert = ProviderList.insertAt(fullProviderList, p2, n - 1);
        if (fullProviderList == insert) {
            return -1;
        }
        Providers.setProviderList(insert);
        return insert.getIndex(name) + 1;
    }
    
    public static int addProvider(Provider provider) {
        return insertProviderAt(provider, 0);
    }
    
    public static synchronized void removeProvider(String s) {
        check("removeProvider." + s);
        Providers.setProviderList(ProviderList.remove(Providers.getFullProviderList(), s));
    }
    
    public static Provider[] getProviders() {
        return Providers.getFullProviderList().toArray();
    }
    
    public static Provider getProvider(String name) {
        return Providers.getProviderList().getProvider(name);
    }
    
    public static Provider[] getProviders(String s) {
        int index = s.indexOf(58);
        String substring;
        String substring2;
        if (index == -1) {
            substring = s;
            substring2 = "";
        }
        else {
            substring = s.substring(0, index);
            substring2 = s.substring(index + 1);
        }
        Hashtable o = new Hashtable(1);
        o.put(substring, substring2);
        return getProviders(o);
    }
    
    public static Provider[] getProviders(Map<String, String> map) {
        Provider[] providers = getProviders();
        Set<Object> keySet = map.keySet();
        LinkedHashSet set = new LinkedHashSet(5);
        if (keySet == null || providers == null) {
            return providers;
        }
        int n = 1;
        for (String s : keySet) {
            LinkedHashSet<Provider> allQualifyingCandidates = getAllQualifyingCandidates(s, (String)map.get(s), providers);
            if (n != 0) {
                set = allQualifyingCandidates;
                n = 0;
            }
            if (allQualifyingCandidates != null && !allQualifyingCandidates.isEmpty()) {
                Iterator<Object> iterator2 = set.iterator();
                while (iterator2.hasNext()) {
                    if (!allQualifyingCandidates.contains((Object)iterator2.next())) {
                        iterator2.remove();
                    }
                    else {
                        continue;
                    }
                }
            }
            else {
                set = null;
                break;
            }
        }
        if (set == null || set.isEmpty()) {
            return null;
        }
        Object[] array = set.toArray();
        Provider[] array2 = new Provider[array.length];
        for (int i = 0; i < array2.length; ++i) {
            array2[i] = (Provider)array[i];
        }
        return array2;
    }
    
    private static Class<?> getSpiClass(String str) {
        Class<Object> clazz = (Class)Security.spiMap.get(str);
        if (clazz != null) {
            return clazz;
        }
        try {
            Class<Object> forName = Class.forName("java.security." + str + "Spi");
            Security.spiMap.put(str, forName);
            return forName;
        }
        catch (ClassNotFoundException cause) {
            throw new AssertionError("Spi class not found", cause);
        }
    }
    
    static Object[] getImpl(String s, String s2, String provider) throws NoSuchAlgorithmException, NoSuchProviderException {
        if (provider == null) {
            return GetInstance.getInstance(s2, getSpiClass(s2), s).toArray();
        }
        return GetInstance.getInstance(s2, getSpiClass(s2), s, provider).toArray();
    }
    
    static Object[] getImpl(String s, String s2, String provider, Object o) throws NoSuchAlgorithmException, NoSuchProviderException, InvalidAlgorithmParameterException {
        if (provider == null) {
            return GetInstance.getInstance(s2, getSpiClass(s2), s, o).toArray();
        }
        return GetInstance.getInstance(s2, getSpiClass(s2), s, o, provider).toArray();
    }
    
    static Object[] getImpl(String algorithm, String type, Provider provider) throws NoSuchAlgorithmException {
        return GetInstance.getInstance(type, getSpiClass(type), algorithm, provider).toArray();
    }
    
    static Object[] getImpl(String algorithm, String type, Provider provider, Object param) throws NoSuchAlgorithmException, InvalidAlgorithmParameterException {
        return GetInstance.getInstance(type, getSpiClass(type), algorithm, param, provider).toArray();
    }
    
    public static String getProperty(String s) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new SecurityPermission("getProperty." + s));
        }
        String s2 = Security.props.getProperty(s);
        if (s2 != null) {
            s2 = s2.trim();
            return s2;
        }
        return s2;
    }
    
    public static void setProperty(String key, String s) {
        check("setProperty." + key);
        Security.props.put(key, s);
        invalidateSMCache(key);
        if (EventHelper.isLoggingSecurity()) {
            EventHelper.logSecurityPropertyEvent(key, s);
        }
    }
    
    private static void invalidateSMCache(String s) {
        final int equals = s.equals("package.access");
        int equals2 = s.equals("package.definition");
        if (equals || equals2) {
            AccessController.doPrivileged(new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    try {
                        Class<Object> forName = Class.forName("java.lang.SecurityManager", false, null);
                        Field obj;
                        int accessible;
                        if (!(equals == 0)) {
                            obj = forName.getDeclaredField("packageAccessValid");
                            accessible = obj.isAccessible();
                            obj.setAccessible(true);
                        }
                        else {
                            obj = forName.getDeclaredField("packageDefinitionValid");
                            accessible = obj.isAccessible();
                            obj.setAccessible(true);
                        }
                        obj.setBoolean(obj, false);
                        obj.setAccessible((boolean)accessible);
                    }
                    catch (Exception ex) {}
                    return null;
                }
            });
        }
    }
    
    private static void check(String target) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkSecurityAccess(target);
        }
    }
    
    private static void checkInsertProvider(String str) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            try {
                securityManager.checkSecurityAccess("insertProvider");
            }
            catch (SecurityException ex) {
                try {
                    securityManager.checkSecurityAccess("insertProvider." + str);
                }
                catch (SecurityException exception) {
                    ex.addSuppressed(exception);
                    throw ex;
                }
            }
        }
    }
    
    private static LinkedHashSet<Provider> getAllQualifyingCandidates(String s, String s2, Provider[] array) {
        String[] filterComponents = getFilterComponents(s, s2);
        return getProvidersNotUsingCache(filterComponents[0], filterComponents[1], filterComponents[2], s2, array);
    }
    
    private static LinkedHashSet<Provider> getProvidersNotUsingCache(String s, String s2, String s3, String s4, Provider[] array) {
        LinkedHashSet set = new LinkedHashSet(5);
        for (int i = 0; i < array.length; ++i) {
            if (isCriterionSatisfied(array[i], s, s2, s3, s4)) {
                set.add(array[i]);
            }
        }
        return set;
    }
    
    private static boolean isCriterionSatisfied(Provider provider, String str, String s, String s2, String s3) {
        String str2 = str + 46 + s;
        if (s2 != null) {
            str2 = str2 + 32 + s2;
        }
        String anotherString = getProviderProperty(str2, provider);
        if (anotherString == null) {
            String providerProperty = getProviderProperty("Alg.Alias." + str + "." + s, provider);
            if (providerProperty != null) {
                String str3 = str + "." + providerProperty;
                if (s2 != null) {
                    str3 = str3 + 32 + s2;
                }
                anotherString = getProviderProperty(str3, provider);
            }
            if (anotherString == null) {
                return false;
            }
        }
        if (s2 == null) {
            return true;
        }
        if (isStandardAttr(s2)) {
            return isConstraintSatisfied(s2, s3, anotherString);
        }
        return s3.equalsIgnoreCase(anotherString);
    }
    
    private static boolean isStandardAttr(String s) {
        return s.equalsIgnoreCase("KeySize") || s.equalsIgnoreCase("ImplementedIn");
    }
    
    private static boolean isConstraintSatisfied(String s, String s2, String s3) {
        if (s.equalsIgnoreCase("KeySize")) {
            return Integer.parseInt(s2) <= Integer.parseInt(s3);
        }
        return s.equalsIgnoreCase("ImplementedIn") && s2.equalsIgnoreCase(s3);
    }
    
    static String[] getFilterComponents(String s, String s2) {
        int index = s.indexOf(46);
        if (index < 0) {
            throw new InvalidParameterException("Invalid filter");
        }
        String substring = s.substring(0, index);
        String trim = null;
        String s3;
        if (s2.length() == 0) {
            s3 = s.substring(index + 1).trim();
            if (s3.length() == 0) {
                throw new InvalidParameterException("Invalid filter");
            }
        }
        else {
            int index2 = s.indexOf(32);
            if (index2 == -1) {
                throw new InvalidParameterException("Invalid filter");
            }
            trim = s.substring(index2 + 1).trim();
            if (trim.length() == 0) {
                throw new InvalidParameterException("Invalid filter");
            }
            if (index2 < index || index == index2 - 1) {
                throw new InvalidParameterException("Invalid filter");
            }
            s3 = s.substring(index + 1, index2);
        }
        return new String[] { substring, s3, trim };
    }
    
    public static Set<String> getAlgorithms(String s) {
        if (s == null || s.length() == 0 || s.endsWith(".")) {
            return Collections.emptySet();
        }
        HashSet<String> s2 = new HashSet();
        Provider[] providers = getProviders();
        for (int i = 0; i < providers.length; ++i) {
            Enumeration<Object> keys = providers[i].keys();
            while (keys.hasMoreElements()) {
                String upperCase = ((String)keys.nextElement()).toUpperCase(Locale.ENGLISH);
                if (upperCase.startsWith(s.toUpperCase(Locale.ENGLISH)) && upperCase.indexOf(" ") < 0) {
                    s2.add(upperCase.substring(s.length() + 1));
                }
                else {
                    continue;
                }
            }
        }
        return Collections.unmodifiableSet(s2);
    }
    
    static void access$000() {
        initialize();
    }
    
    static {
        sdebug = Debug.getInstance("properties");
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                Security.access$000();
                return null;
            }
        });
        spiMap = new ConcurrentHashMap();
    }
    
    private static class ProviderProperty
    {
        String className;
        Provider provider;
    }
}
